@@ -309,6 +309,8 @@ class DynamicDataSpecFunc(NodeFunction[_data.DataSpec]):
 
     def _execute(self, ctx: NodeContext) -> _data.DataSpec:
 
+        # When data def for an output was not supplied in the job, this function creates a dynamic data spec
+
         if self.node.prior_data_spec is not None:
             raise _ex.ETracInternal("Data updates not supported yet")
 
@@ -370,6 +372,8 @@ class DynamicDataSpecFunc(NodeFunction[_data.DataSpec]):
         storage_def = meta.StorageDefinition()
         storage_def.dataItems[data_item] = storage_item
 
+        # Dynamic data def will always use an embedded schema (this is no ID for an external schema)
+
         return _data.DataSpec(
             data_item,
             data_def,
@@ -507,10 +511,11 @@ class ImportModelFunc(NodeFunction[meta.ObjectDefinition]):
 
 class RunModelFunc(NodeFunction[Bundle[_data.DataView]]):
 
-    def __init__(self, node: RunModelNode, model_class: _api.TracModel.__class__):
+    def __init__(self, node: RunModelNode, model_class: _api.TracModel.__class__, checkout_directory: pathlib.Path):
         super().__init__()
         self.node = node
         self.model_class = model_class
+        self.checkout_directory = checkout_directory
 
     def _execute(self, ctx: NodeContext) -> Bundle[_data.DataView]:
 
@@ -558,7 +563,8 @@ class RunModelFunc(NodeFunction[Bundle[_data.DataView]]):
         except _ex.ETrac:
             raise
         except Exception as e:
-            msg = f"There was an unhandled error in the model: {str(e)}"
+            details = _util.error_details_from_model_exception(e, self.checkout_directory)
+            msg = f"There was an unhandled error in the model: {str(e)}{details}"
             raise _ex.EModelExec(msg) from e
 
         # The node result is just the model outputs taken from the local context
@@ -624,10 +630,11 @@ class FunctionResolver:
     def resolve_run_model_node(self, node: RunModelNode) -> NodeFunction:
 
         model_class = self._models.load_model_class(node.model_scope, node.model_def)
+        checkout_directory = self._models.model_load_checkout_directory(node.model_scope, node.model_def)
 
         # TODO: Verify model_class against model_def
 
-        return RunModelFunc(node, model_class)
+        return RunModelFunc(node, model_class, checkout_directory)
 
     __basic_node_mapping: tp.Dict[Node.__class__, NodeFunction.__class__] = {
 
@@ -642,7 +649,8 @@ class FunctionResolver:
         DataResultNode: DataResultFunc,
         StaticValueNode: StaticValueFunc,
         BundleItemNode: NoopFunc,
-        NoopNode: NoopFunc
+        NoopNode: NoopFunc,
+        RunModelResultNode: NoopFunc
     }
 
     __node_mapping: tp.Dict[Node.__class__, __ResolveFunc] = {
