@@ -357,6 +357,15 @@ class RunModelNode(Node[Bundle[_data.DataView]]):
         return {dep_id: DependencyType.HARD for dep_id in [*self.parameter_ids, *self.input_ids]}
 
 
+@_node_type
+class RunModelResultNode(Node[None]):
+
+    model_id: NodeId
+
+    def _node_dependencies(self) -> tp.Dict[NodeId, DependencyType]:
+        return {self.model_id: DependencyType.HARD}
+
+
 @_node_type
 class BuildJobResultNode(Node[cfg.JobResult]):
 
