@@ -160,17 +160,17 @@ class GraphBuilder:
 
         input_section = cls.build_job_inputs(
             job_config, job_namespace, inputs,
-            explicit_deps=[job_push_id, *params_section.must_run])
+            explicit_deps=[job_push_id])
 
         exec_obj = _util.get_job_resource(target, job_config)
 
         exec_section = cls.build_model_or_flow(
             job_config, job_namespace, exec_obj,
-            explicit_deps=[job_push_id, *params_section.must_run, *input_section.must_run])
+            explicit_deps=[job_push_id])
 
         output_section = cls.build_job_outputs(
             job_config, job_namespace, outputs,
-            explicit_deps=[job_push_id, *exec_section.must_run])
+            explicit_deps=[job_push_id])
 
         main_section = cls._join_sections(params_section, input_section, exec_section, output_section)
 
@@ -223,9 +223,14 @@ class GraphBuilder:
             data_def = _util.get_job_resource(data_selector, job_config).data
             storage_def = _util.get_job_resource(data_def.storageId, job_config).storage
 
+            if data_def.schemaId:
+                schema_def = _util.get_job_resource(data_def.schemaId, job_config).schema
+            else:
+                schema_def = data_def.schema
+
             root_part_opaque_key = 'part-root'  # TODO: Central part names / constants
             data_item = data_def.parts[root_part_opaque_key].snap.deltas[0].dataItem
-            data_spec = _data.DataSpec(data_item, data_def, storage_def, schema_def=None)
+            data_spec = _data.DataSpec(data_item, data_def, storage_def, schema_def)
 
             # Data spec node is static, using the assembled data spec
             data_spec_id = NodeId.of(f"{input_name}:SPEC", job_namespace, _data.DataSpec)
@@ -238,7 +243,7 @@ class GraphBuilder:
 
             # Input views assembled by mapping one root part to each view
             data_view_id = NodeId.of(input_name, job_namespace, _data.DataView)
-            data_view_node = DataViewNode(data_view_id, data_def.schema, data_load_id)
+            data_view_node = DataViewNode(data_view_id, schema_def, data_load_id)
 
             nodes[data_spec_id] = data_spec_node
             nodes[data_load_id] = data_load_node
@@ -270,13 +275,19 @@ class GraphBuilder:
 
             if data_obj is not None:
 
+                # If data def for the output has been built in advance, use a static data spec
+
                 data_def = data_obj.data
-                storage_obj = _util.get_job_resource(data_def.storageId, job_config)
-                storage_def = storage_obj.storage
+                storage_def = _util.get_job_resource(data_def.storageId, job_config).storage
+
+                if data_def.schemaId:
+                    schema_def = _util.get_job_resource(data_def.schemaId, job_config).schema
+                else:
+                    schema_def = data_def.schema
 
                 root_part_opaque_key = 'part-root'  # TODO: Central part names / constants
                 data_item = data_def.parts[root_part_opaque_key].snap.deltas[0].dataItem
-                data_spec = _data.DataSpec(data_item, data_def, storage_def, schema_def=None)
+                data_spec = _data.DataSpec(data_item, data_def, storage_def, schema_def)
 
                 data_spec_node = StaticValueNode(data_spec_id, data_spec, explicit_deps=explicit_deps)
 
@@ -285,6 +296,9 @@ class GraphBuilder:
 
             else:
 
+                # If output data def for an output was not supplied in the job, create a dynamic data spec
+                # Dynamic data def will always use an embedded schema (this is no ID for an external schema)
+
                 data_key = output_name + ":DATA"
                 data_id = job_config.resultMapping[data_key]
                 storage_key = output_name + ":STORAGE"
@@ -377,6 +391,7 @@ class GraphBuilder:
         sub_namespace = NodeNamespace(sub_namespace_name, namespace)
 
         # Execute in the sub-context by doing PUSH, EXEC, POP
+        # Note that POP node must be in the sub namespace too
 
         push_section = cls.build_context_push(
             sub_namespace, input_mapping,
@@ -443,7 +458,10 @@ class GraphBuilder:
             frozenset(parameter_ids), frozenset(input_ids),
             explicit_deps=explicit_deps, bundle=model_id.namespace)
 
-        nodes = {model_id: model_node}
+        module_result_id = NodeId(f"{model_name}:RESULT", namespace)
+        model_result_node = RunModelResultNode(module_result_id, model_id)
+
+        nodes = {model_id: model_node, module_result_id: model_result_node}
 
         # Create nodes for each model output
         # The model node itself outputs a bundle (dictionary of named outputs)
@@ -456,7 +474,7 @@ class GraphBuilder:
             nodes[output_id] = BundleItemNode(output_id, model_id, output_id.name)
 
         # Assemble a graph to include the model and its outputs
-        return GraphSection(nodes, inputs={*parameter_ids, *input_ids}, outputs=output_ids, must_run=[model_id])
+        return GraphSection(nodes, inputs={*parameter_ids, *input_ids}, outputs=output_ids, must_run=[module_result_id])
 
     @classmethod
     def build_flow(
@@ -516,7 +534,10 @@ class GraphBuilder:
                         reachable_nodes[target_node_name] = target_node
 
         if any(remaining_nodes):
-            raise _ex.ETracInternal()  # todo: cyclic / unmet dependencies
+            missing_targets = [edge.target for node in remaining_edges_by_target.values() for edge in node]
+            missing_target_names = [f"{t.node}.{t.socket}" if t.socket else t.node for t in missing_targets]
+            missing_nodes = list(map(lambda n: NodeId(n, namespace), missing_target_names))
+            cls._invalid_graph_error(missing_nodes)
 
         return graph_section
 
@@ -648,7 +669,7 @@ class GraphBuilder:
 
         nodes = {**first_section.nodes}
         inputs = set(first_section.inputs)
-        must_run = first_section.must_run or []
+        must_run = list(first_section.must_run) if first_section.must_run else []
 
         for i in range(1, n_sections):
 
@@ -662,10 +683,33 @@ class GraphBuilder:
                 if allow_partial_inputs:
                     inputs.update(requirements_not_met)
                 else:
-                    err = ', '.join(map(str, requirements_not_met))
-                    raise _ex.ETracInternal(err)  # todo inconsistent graph
+                    cls._invalid_graph_error(requirements_not_met)
 
             nodes.update(current_section.nodes)
+
+            must_run = list(filter(lambda n: n not in current_section.inputs, must_run))
             must_run.extend(current_section.must_run)
 
         return GraphSection(nodes, inputs, last_section.outputs, must_run)
+
+    @classmethod
+    def _invalid_graph_error(cls, missing_dependencies: tp.Iterable[NodeId]):
+
+        missing_ids = ", ".join(map(cls._missing_item_display_name, missing_dependencies))
+        message = f"Invalid job config: The execution graph has unsatisfied dependencies: [{missing_ids}]"
+
+        raise _ex.EJobValidation(message)
+
+    @classmethod
+    def _missing_item_display_name(cls, node_id: NodeId):
+
+        components = node_id.namespace.components()
+
+        # The execution graph is built for an individual job, so the top-level namespace is always the job key
+        # Do not list the job key component with every missing node, that would be overly verbose
+        # If we ever start using global resources (to share between jobs), this logic may need to change
+
+        if len(components) <= 1:
+            return node_id.name
+        else:
+            return f"{node_id.name} / {', '.join(components[:-1])}"
