@@ -35,6 +35,7 @@ import tracdap.rt._impl.util as _util  # noqa
 import tracdap.rt._impl.models as _models  # noqa
 import tracdap.rt._impl.storage as _storage  # noqa
 import tracdap.rt._impl.static_api as _static_api  # noqa
+import tracdap.rt._impl.guard_rails as _guard  # noqa
 import tracdap.rt._version as _version
 
 
@@ -47,6 +48,12 @@ class _RuntimeJobInfo:
 
 class TracRuntime:
 
+    # Basic configuration of engine threads
+    __THREAD_POOL_CONFIG = {"model": 1, "data": 3}
+    __THREAD_POOL_MAPPING = {
+        _engine.ModelNodeProcessor: "model",
+        _engine.DataNodeProcessor: "data"}
+
     def __init__(
             self,
             sys_config: tp.Union[str, pathlib.Path, _cfg.RuntimeConfig],
@@ -125,6 +132,7 @@ class TracRuntime:
 
             _plugins.PluginManager.register_core_plugins()
             _static_api.StaticApiImpl.register_impl()
+            _guard.PythonGuardRails.protect_dangerous_functions()
 
             # Load sys config (or use embedded), config errors are detected before start()
             # Job config can also be checked before start() by using load_job_config()
@@ -160,7 +168,11 @@ class TracRuntime:
                 self._sys_config, self._models, self._storage,
                 notify_callback=self._engine_callback)
 
-            self._system = _actors.ActorSystem(self._engine, system_thread="engine")
+            self._system = _actors.ActorSystem(
+                self._engine, system_thread="engine",
+                thread_pools=self.__THREAD_POOL_CONFIG,
+                thread_pool_mapping=self.__THREAD_POOL_MAPPING)
+
             self._system.start(wait=wait)
 
         except Exception as e:
